import { Ciphertext, Digest, Fields, Id, Secret } from './types';
import { CiphertextAndDigest } from './events';
export interface SecretIdent {
    fromAddr: string;
    toAddr: string | undefined;
    randomId: Id;
    index: number;
}
export declare abstract class RandomSpec {
    abstract asOptions(): string[];
}
export declare class ShuffledList extends RandomSpec {
    options: string[];
    constructor(fields: {
        options: string[];
    });
    asOptions(): string[];
}
export declare class Lottery extends RandomSpec {
    optionsAndWeights: Map<string, number>;
    constructor(fields: Fields<Lottery>);
    asOptions(): string[];
}
export declare class Lock {
    digest: Digest;
    owner: string;
    constructor(owner: string, digest: Digest);
}
export type MaskStatus = 'required' | 'applied' | 'removed';
export declare class Mask {
    status: MaskStatus;
    readonly owner: string;
    constructor(owner: string);
}
export type CipherOwner = {
    kind: 'unclaimed';
} | {
    kind: 'assigned';
    addr: string;
} | {
    kind: 'multiAssigned';
    addrs: string[];
} | {
    kind: 'revealed';
};
export declare class LockedCiphertext {
    locks: Lock[];
    owner: CipherOwner;
    ciphertext: Ciphertext;
    constructor(text: Ciphertext);
}
export declare class Share {
    fromAddr: string;
    toAddr: string | undefined;
    index: number;
    secret: Secret | undefined;
    constructor(fromAddr: string, index: number, toAddr?: string);
}
export type RandomStatus = {
    kind: 'ready';
} | {
    kind: 'locking';
    addr: string;
} | {
    kind: 'masking';
    addr: string;
} | {
    kind: 'waiting-secrets';
} | {
    kind: 'shared';
};
export declare class RandomState {
    id: Id;
    size: number;
    owners: string[];
    options: string[];
    status: RandomStatus;
    masks: Mask[];
    ciphertexts: LockedCiphertext[];
    secretShares: Share[];
    revealed: Map<number, string>;
    constructor(id: Id, spec: RandomSpec, owners: string[]);
    isFullyMasked(): boolean;
    isFullyLocked(): boolean;
    mask(addr: string, ciphertexts: Ciphertext[]): void;
    lock(addr: string, ciphertextsAndDigests: CiphertextAndDigest[]): void;
    assign(addr: string, indexes: number[]): void;
    addSecretShare(share: Share): void;
    reveal(indexes: number[]): void;
    listRequiredSecretsByFromAddr(fromAddr: string): SecretIdent[];
    listRevealedSecrets(): Map<number, Secret[]>;
    listAssignedCiphertexts(addr: string): Map<number, Ciphertext>;
    listRevealedCiphertexts(): Map<number, Ciphertext>;
    listSharedSecrets(toAddr: string): Map<number, Secret[]>;
    addRevealed(revealed: Map<number, string>): void;
    addSecret(fromAddr: string, toAddr: string | undefined, index: number, secret: Secret): void;
    listOperatingAddrs(): string[];
    updateStatus(): void;
}
