"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RandomState = exports.Share = exports.LockedCiphertext = exports.Mask = exports.Lock = exports.Lottery = exports.ShuffledList = exports.RandomSpec = void 0;
const borsh_1 = require("@race-foundation/borsh");
class RandomSpec {
}
exports.RandomSpec = RandomSpec;
let ShuffledList = class ShuffledList extends RandomSpec {
    constructor(fields) {
        super();
        Object.assign(this, fields);
    }
    asOptions() {
        return this.options;
    }
};
__decorate([
    (0, borsh_1.field)((0, borsh_1.array)('string'))
], ShuffledList.prototype, "options", void 0);
ShuffledList = __decorate([
    (0, borsh_1.variant)(0)
], ShuffledList);
exports.ShuffledList = ShuffledList;
let Lottery = class Lottery extends RandomSpec {
    constructor(fields) {
        super();
        Object.assign(this, fields);
    }
    asOptions() {
        const res = [];
        for (const [k, v] of this.optionsAndWeights) {
            for (let i = 0; i < v; i++) {
                res.push(k);
            }
        }
        return res;
    }
};
__decorate([
    (0, borsh_1.field)((0, borsh_1.map)('string', 'u16'))
], Lottery.prototype, "optionsAndWeights", void 0);
Lottery = __decorate([
    (0, borsh_1.variant)(1)
], Lottery);
exports.Lottery = Lottery;
class Lock {
    constructor(owner, digest) {
        this.digest = digest;
        this.owner = owner;
    }
}
exports.Lock = Lock;
class Mask {
    constructor(owner) {
        this.owner = owner;
        this.status = 'required';
    }
}
exports.Mask = Mask;
class LockedCiphertext {
    constructor(text) {
        this.ciphertext = text;
        this.locks = [];
        this.owner = { kind: 'unclaimed' };
    }
}
exports.LockedCiphertext = LockedCiphertext;
class Share {
    constructor(fromAddr, index, toAddr) {
        this.fromAddr = fromAddr;
        this.index = index;
        this.toAddr = toAddr;
        this.secret = undefined;
    }
}
exports.Share = Share;
class RandomState {
    constructor(id, spec, owners) {
        if (owners.length === 0) {
            throw new Error('No enough servers');
        }
        this.owners = owners;
        const options = spec.asOptions();
        const ciphertexts = options.map(o => new LockedCiphertext(new TextEncoder().encode(o)));
        this.options = options;
        this.size = options.length;
        this.ciphertexts = ciphertexts;
        this.masks = owners.map(o => new Mask(o));
        this.id = id;
        this.revealed = new Map();
        this.secretShares = [];
        this.status = { kind: 'masking', addr: owners[0] };
    }
    isFullyMasked() {
        return this.masks.every(m => m.status !== 'required');
    }
    isFullyLocked() {
        return this.masks.every(m => m.status === 'removed');
    }
    mask(addr, ciphertexts) {
        if (this.status.kind === 'masking') {
            const a = this.status.addr;
            if (a !== addr) {
                throw new Error('Invalid operator');
            }
            const mask = this.masks.find(m => m.owner === addr);
            if (mask === undefined) {
                throw new Error('Invalid operator');
            }
            if (mask.status !== 'required') {
                throw new Error('Duplicated mask');
            }
            if (ciphertexts.length !== this.ciphertexts.length) {
                throw new Error('Invalid ciphertexts');
            }
            for (let i = 0; i < this.ciphertexts.length; i++) {
                this.ciphertexts[i].ciphertext = ciphertexts[i];
            }
            mask.status = 'applied';
            this.updateStatus();
        }
        else {
            throw new Error('Invalid status' + this.status);
        }
    }
    lock(addr, ciphertextsAndDigests) {
        if (this.status.kind === 'locking') {
            const a = this.status.addr;
            if (a !== addr) {
                throw new Error('Invalid operator');
            }
            const mask = this.masks.find(m => m.owner === addr);
            if (mask === undefined) {
                throw new Error('Invalid operator');
            }
            if (ciphertextsAndDigests.length !== this.ciphertexts.length) {
                throw new Error('Invalid ciphertexts');
            }
            mask.status = 'removed';
            for (let i = 0; i < this.ciphertexts.length; i++) {
                const { ciphertext, digest } = ciphertextsAndDigests[i];
                this.ciphertexts[i].ciphertext = ciphertext;
                this.ciphertexts[i].locks.push(new Lock(addr, digest));
            }
            this.updateStatus();
        }
        else {
            throw new Error('Invalid status' + this.status);
        }
    }
    assign(addr, indexes) {
        if (this.status.kind === 'ready' || this.status.kind === 'shared' || this.status.kind === 'waiting-secrets') {
            for (const idx of indexes) {
                let c = this.ciphertexts[idx];
                if (c.owner.kind === 'assigned' || c.owner.kind === 'revealed') {
                    throw new Error('Ciphertext already assigned');
                }
                c.owner = { kind: 'assigned', addr };
                let secrets = this.secretShares;
                this.owners.forEach(o => {
                    secrets.push(new Share(o, idx, addr));
                });
            }
            this.status = { kind: 'waiting-secrets' };
        }
        else {
            throw new Error('Invalid status' + this.status);
        }
    }
    addSecretShare(share) {
        const exist = this.secretShares.find(ss => ss.fromAddr === share.fromAddr && ss.toAddr === share.toAddr && ss.index === share.index);
        if (exist === undefined) {
            this.secretShares.push(share);
        }
    }
    reveal(indexes) {
        if (this.status.kind === 'ready' || this.status.kind === 'shared' || this.status.kind === 'waiting-secrets') {
            for (const idx of indexes) {
                let c = this.ciphertexts[idx];
                if (c.owner.kind !== 'revealed') {
                    c.owner = { kind: 'revealed' };
                    let secrets = this.secretShares;
                    this.owners.forEach(o => {
                        secrets.push(new Share(o, idx));
                    });
                }
            }
            this.status = { kind: 'waiting-secrets' };
        }
        else {
            throw new Error('Invalid status' + this.status);
        }
    }
    listRequiredSecretsByFromAddr(fromAddr) {
        return this.secretShares
            .filter(ss => ss.secret === undefined && ss.fromAddr === fromAddr)
            .map(ss => ({
            fromAddr: ss.fromAddr,
            toAddr: ss.toAddr,
            randomId: this.id,
            index: ss.index,
        }));
    }
    listRevealedSecrets() {
        if (this.status.kind !== 'ready') {
            throw new Error('Secrets not ready, current status: ' + this.status.kind);
        }
        let res = new Map();
        for (const ss of this.secretShares) {
            if (ss.toAddr === undefined) {
                let ciphertexts = res.get(ss.index);
                if (ciphertexts === undefined) {
                    res.set(ss.index, [ss.secret]);
                }
                else {
                    ciphertexts.push(ss.secret);
                }
            }
        }
        return res;
    }
    listAssignedCiphertexts(addr) {
        let res = new Map();
        for (let i = 0; i < this.ciphertexts.length; i++) {
            const c = this.ciphertexts[i];
            if (c.owner.kind === 'assigned' && c.owner.addr === addr) {
                res.set(i, c.ciphertext);
            }
        }
        return res;
    }
    listRevealedCiphertexts() {
        let res = new Map();
        for (let i = 0; i < this.ciphertexts.length; i++) {
            const c = this.ciphertexts[i];
            if (c.owner.kind === 'revealed') {
                res.set(i, c.ciphertext);
            }
        }
        return res;
    }
    listSharedSecrets(toAddr) {
        if (this.status.kind !== 'ready') {
            throw new Error('Secrets not ready, current status: ' + this.status.kind);
        }
        let res = new Map();
        for (const ss of this.secretShares) {
            if (ss.toAddr === toAddr) {
                let secrets = res.get(ss.index);
                if (secrets === undefined) {
                    res.set(ss.index, [ss.secret]);
                }
                else {
                    secrets.push(ss.secret);
                }
            }
        }
        return res;
    }
    addRevealed(revealed) {
        for (const [index, value] of revealed) {
            if (index >= this.size) {
                throw new Error('Invalid index');
            }
            this.revealed.set(index, value);
        }
    }
    addSecret(fromAddr, toAddr, index, secret) {
        const secretShare = this.secretShares.find(ss => ss.fromAddr === fromAddr && ss.toAddr === toAddr && ss.index === index);
        if (secretShare !== undefined) {
            if (secretShare.secret === undefined) {
                const ciphertext = this.ciphertexts[secretShare.index];
                if (ciphertext !== undefined) {
                    secretShare.secret = secret;
                }
                else {
                    throw new Error('Invalid secret');
                }
            }
            else {
                throw new Error('Duplicated secret');
            }
        }
        this.updateStatus();
    }
    listOperatingAddrs() {
        switch (this.status.kind) {
            case 'ready':
                return [];
            case 'shared':
                return [];
            case 'locking':
                return [this.status.addr];
            case 'masking':
                return [this.status.addr];
            case 'waiting-secrets':
                return this.secretShares.filter(s => s.secret === undefined).map(s => s.fromAddr);
        }
    }
    updateStatus() {
        if (this.status.kind === 'locking' && this.masks.every(m => m.status === 'removed')) {
            this.status = { kind: 'ready' };
            return;
        }
        let mask = this.masks.find(m => m.status === 'required');
        if (mask !== undefined) {
            this.status = { kind: 'masking', addr: mask.owner };
            return;
        }
        mask = this.masks.find(m => m.status === 'applied');
        if (mask !== undefined) {
            this.status = { kind: 'locking', addr: mask.owner };
            return;
        }
        if (this.secretShares.find(s => s.secret === undefined) !== undefined) {
            this.status = { kind: 'waiting-secrets' };
            return;
        }
        this.status = { kind: 'shared' };
    }
}
exports.RandomState = RandomState;
//# sourceMappingURL=random-state.js.map