var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { RandomSpec } from './random-state';
import { HandleError } from './error';
import { enums, field, map, option, struct, variant, array } from '@race-foundation/borsh';
export class SettleOp {
}
let SettleAdd = class SettleAdd extends SettleOp {
    constructor(fields) {
        super();
        Object.assign(this, fields);
    }
};
__decorate([
    field('u64')
], SettleAdd.prototype, "amount", void 0);
SettleAdd = __decorate([
    variant(0)
], SettleAdd);
export { SettleAdd };
let SettleSub = class SettleSub extends SettleOp {
    constructor(fields) {
        super();
        Object.assign(this, fields);
    }
};
__decorate([
    field('u64')
], SettleSub.prototype, "amount", void 0);
SettleSub = __decorate([
    variant(1)
], SettleSub);
export { SettleSub };
let SettleEject = class SettleEject extends SettleOp {
    constructor(_) {
        super();
    }
};
SettleEject = __decorate([
    variant(2)
], SettleEject);
export { SettleEject };
export class Settle {
    constructor(fields) {
        this.addr = fields.addr;
        this.op = fields.op;
    }
    sortKey() {
        if (this.op instanceof SettleAdd) {
            return 0;
        }
        else if (this.op instanceof SettleSub) {
            return 1;
        }
        else {
            return 2;
        }
    }
    compare(s) {
        return this.sortKey() - s.sortKey();
    }
}
__decorate([
    field('string')
], Settle.prototype, "addr", void 0);
__decorate([
    field(enums(SettleOp))
], Settle.prototype, "op", void 0);
export class Ask {
    constructor(fields) {
        Object.assign(this, fields);
    }
}
__decorate([
    field('string')
], Ask.prototype, "playerAddr", void 0);
export class Assign {
    constructor(fields) {
        Object.assign(this, fields);
    }
}
__decorate([
    field('usize')
], Assign.prototype, "randomId", void 0);
__decorate([
    field('string')
], Assign.prototype, "playerAddr", void 0);
__decorate([
    field(array('usize'))
], Assign.prototype, "indexes", void 0);
export class Reveal {
    constructor(fields) {
        Object.assign(this, fields);
    }
}
__decorate([
    field('usize')
], Reveal.prototype, "randomId", void 0);
__decorate([
    field(array('usize'))
], Reveal.prototype, "indexes", void 0);
export class Release {
    constructor(fields) {
        Object.assign(this, fields);
    }
}
__decorate([
    field('usize')
], Release.prototype, "decisionId", void 0);
export class ActionTimeout {
    constructor(fields) {
        Object.assign(this, fields);
    }
}
__decorate([
    field('string')
], ActionTimeout.prototype, "playerAddr", void 0);
__decorate([
    field('u64')
], ActionTimeout.prototype, "timeout", void 0);
export class Effect {
    constructor(fields) {
        Object.assign(this, fields);
    }
    static fromContext(context) {
        const revealed = new Map();
        for (const st of context.randomStates) {
            revealed.set(st.id, st.revealed);
        }
        const answered = new Map();
        for (const st of context.decisionStates) {
            answered.set(st.id, st.value);
        }
        const actionTimeout = undefined;
        const waitTimeout = undefined;
        const startGame = false;
        const stopGame = false;
        const cancelDispatch = false;
        const timestamp = context.timestamp;
        const currRandomId = context.randomStates.length + 1;
        const currDecisionId = context.decisionStates.length + 1;
        const playersCount = context.players.length;
        const serversCount = context.servers.length;
        const asks = [];
        const assigns = [];
        const releases = [];
        const reveals = [];
        const initRandomStates = [];
        const settles = [];
        const handlerState = context.handlerState;
        const error = undefined;
        const allowExit = context.allowExit;
        const checkpoint = false;
        return new Effect({
            actionTimeout,
            waitTimeout,
            startGame,
            stopGame,
            cancelDispatch,
            timestamp,
            currRandomId,
            currDecisionId,
            playersCount,
            serversCount,
            asks,
            assigns,
            releases,
            reveals,
            initRandomStates,
            revealed,
            answered,
            settles,
            handlerState,
            error,
            allowExit,
            checkpoint,
        });
    }
}
__decorate([
    field(option(struct(ActionTimeout)))
], Effect.prototype, "actionTimeout", void 0);
__decorate([
    field(option('u64'))
], Effect.prototype, "waitTimeout", void 0);
__decorate([
    field('bool')
], Effect.prototype, "startGame", void 0);
__decorate([
    field('bool')
], Effect.prototype, "stopGame", void 0);
__decorate([
    field('bool')
], Effect.prototype, "cancelDispatch", void 0);
__decorate([
    field('u64')
], Effect.prototype, "timestamp", void 0);
__decorate([
    field('usize')
], Effect.prototype, "currRandomId", void 0);
__decorate([
    field('usize')
], Effect.prototype, "currDecisionId", void 0);
__decorate([
    field('u16')
], Effect.prototype, "playersCount", void 0);
__decorate([
    field('u16')
], Effect.prototype, "serversCount", void 0);
__decorate([
    field(array(struct(Ask)))
], Effect.prototype, "asks", void 0);
__decorate([
    field(array(struct(Assign)))
], Effect.prototype, "assigns", void 0);
__decorate([
    field(array(struct(Reveal)))
], Effect.prototype, "reveals", void 0);
__decorate([
    field(array(struct(Release)))
], Effect.prototype, "releases", void 0);
__decorate([
    field(array(enums(RandomSpec)))
], Effect.prototype, "initRandomStates", void 0);
__decorate([
    field(map('usize', map('usize', 'string')))
], Effect.prototype, "revealed", void 0);
__decorate([
    field(map('usize', 'string'))
], Effect.prototype, "answered", void 0);
__decorate([
    field(array(struct(Settle)))
], Effect.prototype, "settles", void 0);
__decorate([
    field(option('u8-array'))
], Effect.prototype, "handlerState", void 0);
__decorate([
    field(option(enums(HandleError)))
], Effect.prototype, "error", void 0);
__decorate([
    field('bool')
], Effect.prototype, "allowExit", void 0);
__decorate([
    field('bool')
], Effect.prototype, "checkpoint", void 0);
//# sourceMappingURL=effect.js.map